;************************************************************************
;       file eeprom.inc
;************************************************************************
;
; EEPROM R/W routines (standard)
; Read is called with the address of the position in EEADR
; and returns with the read data in W
; Write is called with the address of the position in EEADR
; and data to be written in W
; During WRITE interrupts will be disabled! The routine returns with the
; same state it was called.
;
;************************************************************************
; Henk Tobbe - VK2GWK - tobbe @bigpond.com. mods by gwr
;************************************************************************
;
;WRITE DATA TO EEPROM ROUTINE:

EE_Write
	btfsc	INTCON,GIE	;is interrupt enabled ?
	goto	EE_interrupt	;yes, take special care
	movwf	EEDATA		;copy W into eeprom data byte register
  BANK1
	bsf	EECON1, WREN	;enable write flag
	movlw	0x55		;manual dictated factors -
	movwf	EECON2		;they cause the action 
	movlw	0xAA		;required by the eeprom       
	movwf	EECON2		;to store the data in EEDATA
				;at the address held by EEADR
	bsf	EECON1,WR	;Set the "perform write'' flag
CHKWRT	btfss	EECON1,EEIF	;wait until bit 4 of EECON1 is set
	goto	CHKWRT		;to signal that write is done
	bcf	EECON1,WREN	;disable write
	bcf 	EECON1, EEIF   	; clear bit 4 of EECON1
  BANK0
  bcf   INTCON, EEIE    ; clear bit 6 of INTCON
	return          	; and return

EE_interrupt
	bcf	INTCON,GIE	;disable interrupts before writing
	call	EE_Write
  bsf INTCON,GIE  ;enable again
	return


;
;READ DATA FROM EEPROM ROUTINE:

                        
EE_Read
  BANK1
	BSF 	EECON1,RD	; enable read flag
  BANK0
	movf 	EEDATA,W	; read eeprom data now in EEDATA into W 
	return			; and return

;***********************************************************************
; get_EElat, reads block of 4 bytes from EE_prom to address in "ramadr".
; Pointer to 1st EE_prom address is in W when called
; gwr
;
get_EElat
	movwf   EEADR
	movlw   (ramadr)	; Start of Ram Data to save
  movwf   FSR
	movlw   d'4'		; Number of bytes to store
	movwf   counter
get_EElatl
	call    EE_Read
  movwf   INDF
  incf    FSR, f
	incf    EEADR, f
	decfsz  counter, f
	goto    get_EElatl
	return

;***********************************************************************
; put_EElat, writes block of 4 bytes from address in "ramadr" to EE_prom
; pointer to 1st EE_prom address is in W when called
; gwr
put_EElat
	movwf   EEADR
	movlw   (ramadr)	; Start of Ram Data to save
  movwf   FSR
	movlw   d'4'		; Number of bytes to store
	movwf   counter
put_EElatl
  movfw   INDF
	call    EE_Write
  incf    FSR, f
	incf    EEADR, f
	decfsz  counter, f
	goto    put_EElatl
	return

