;*********************************************************************************
;                                                                     
;	12-bit A/D converter - LTC1298
;	==============================
;
;	Writen by: Andric Dragan, 21.07.1999, mikroElektronika for 1286
; modified for LTC1298 by GWR 20-FEB-2002
;	
;
;		
;
;   ADC_LO - LSB data result of AD conversion
;   ADC_HI - MSB data result of AD conversion
;   ADC_Var    - on input:  BIT7 = 0 Channel 0
;                       BIT7 = 1 Channel 1
;          - und internal Loop counter = 0 on exit
;
;	In the main program you must have the next declaration:
;	=======================================================
;	Cblock 0x0C	; Beginning of RAM
; ADC_LO   ; LSB data buffer
; ADC_HI   ; MSB data buffer
; ADC_Var   ; Loop counter
;	endc
;
;       #define	Data	PORTB,4
;       #define	Clock   PORTB,1
;       #define	CS	PORTA,4
;       
;*********************************************************************************

LTC98
  clrf  ADC_LO  ; Erase data buffer
  clrf  ADC_HI
;
  bcf Clock
	bsf	Data
	BANK1
	movlw	0x01
  	movwf 	TRISB
	BANK0
;
  	bcf 	CS  	; Enable AD coverter
  	call  	CLK 	; out START
  	call	CLK	; out SGL
  	btfss 	ADC_Var,7 ; if set channel 1
	bcf	Data	; else channel 0
	call	CLK	; Out channel #
	bsf     Clock	; out MSB
	nop
	BANK1
	movlw	0x11
	movwf	TRISB
	BANK0
  bcf   Clock
	call	CLK	; NULL bit

	movlw   .4
  movwf   ADC_Var ; Counter initialization

ADC_Loop
  rlf ADC_HI,f ; Rotate 'ADC_HI' one space to left

  btfss	Data	 ; is Dout line = '1' ?
  bcf	ADC_HI,0 ; if not, clear bit '0' in Var_HI variable
  btfsc	Data	 ; is Dout line = '0' ?
  bsf	ADC_HI,0 ; if not, set bit '0' in Var_HI variable

  call	CLK	

  decfsz  ADC_Var,f ; Are 4 bits received ?
  goto    ADC_Loop  ; If not, repeat

  movlw   .8
  movwf   ADC_Var ; Counter initialization

ADC_Loop1
  rlf	ADC_LO,f ; Rotate 'ADC_LO' one space to left

  btfss	Data	 ; is Dout line = '1' ?
  bcf	ADC_LO,0 ; if not, clear bit '0' in ADC_LO variable
  btfsc	Data	 ; is Dout line = '0' ?
  bsf	ADC_LO,0 ; if not, set bit '0' in ADC_LO variable
	
  call	CLK

  decfsz  ADC_Var,f ; Are 8 bits received ?
  goto    ADC_Loop1 ; If not, repeat

  bsf	CS	; Disable AD converter
  return

CLK
	bsf     Clock	; Make one clock
  nop
  bcf     Clock
	return

