;*********************************************************************************
;                                                                     
;	Send a series of data on one of output devices such as: LCD, RS232, etc.
;	========================================================================
;	
;	Writen by: Andric Dragan, 21.07.1999, mikroElektronika
;	
;	==============
;	Processor:		Microchip PIC 16XXX
;	Clock:			4.00 MHz XT
;	Instruction cycle:	1 MIPS
;
;
;	List of macros:
;
;		"Print Addr, Start, End, Var, Out"
;
;	Addr  - address where one or more sequences begin
;	Start - address of the first member of the sequence
;	End   - address where the sequence ends
;	Var   - variable which has a role of showing (pointing) 
;		the members of the sequence
;	Out   -	is an argument we use to send the address of existing 
;		subprograms in working with output devices such as: 
;		LCD, RS-232 etc. 
;
;
;	In the main program you must have the next declaration:
;	=======================================================
;	Cblock 0x0C	; Beginning of RAM
;	Pointer		; Pointer on characters in message
;	endc
;
;*********************************************************************************

PRINT macro Addr, Start, End, Var, Out

	Local	Next	; Local label
	Local	Exit

	movlw	Start	; Address of the first member in the series
	movwf	Var	
Next
	movf	Var,w	; Pointer -> W reg.
	call	Addr	; Series(W reg.) -> W reg.
        			; Data is in W reg.
	Out	        	; Display data on the output device
	movf	Var,w 
	xorlw	End-1	  
        btfsc   STATUS,Z ; Is it the end of the series ?
        goto    Exit	 ; If it is, get out of the loop
	incf	Var,f 	 ; If isn`t, set the Pointer to next member
			 ; of the series
	goto	Next	 ; Repeat the loop
Exit
	endm
