;_______________________________________________________________________*
;									*
;       Rotor-Interface using PIC 16f84					*
;       20-feb-2002
;	english language comments added NOV-2002						*
;									*
;       LCD in nibble mode and wait for busyflag			*
;       RS232 via interrupt on PB0 and FIFO Buffer			*
;       A/D-converter LTC1298						*
;									*
;                                                                       *
; Port Configurations :                                                 *
;                                                                       *
; RB0 : Serial Input  (19200 bps 8N1)	nur in                          *
; RB1 : RS-LCD und CLK A/D LTC 1298     nur out                         *
; RB2 : R/_W LCD                        nur out                         *
; RB3 : Enable LCD Data                 nur out                         *
; RB4 : DI/DO LTC 1298                  in/out                          *
; RB4 : LCD (B4) LSB                    in/out                          *
; RB5 : LCD (B5)                          "                             *
; RB6 : LCD (B6)                          "                             *
; RB7 : LCD (B7) MSB                      "                             *
;                                                                       *
; RA0 : Rotor UP                        nur out                         *
; RA1 : Rotor DOWN                      nur out                         *
; RA2 : Rotor LEFT                      nur out                         *
; RA3 : Rotor RIGHT                     nur out                         *
; RA4 : CS_ LTC 1298                    nur out                         *
;
;************************************************************************
;************************************************************************
; processor pic16f84                                                    *
; no code protection                                                    *
; power up timer enabled                                                *
; WDT disabled; gwr                                                     *
; XT oscillator  (4 MHz)                                                *
;************************************************************************
;***** Declaring and configuring a microcontroller *****

        PROCESSOR 16f84
        #include "p16f84.inc"
	
	__CONFIG _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC

;***** Declaring variables *****

	Cblock 0x0C	; Beginning of RAM
; interrupt context save 
        w_
        status_
	fsr_
	pclath_
; LCD
	LCDbuf
	LCDtemp
; Print
        Pointer         ; Pointer on characters in message
; eeprom
        counter
; display 16bit binary value decimal on LCD and math16 routines
        TEMP0
        TEMP1
	TEMP2
	TEMP3
        LO
        HI
        LO_TEMP
	HI_TEMP
	HI_RESULT
	Bcheck
; A/D LTC1298
	ADC_Var
	ADLO_sav
	ADHI_sav
; RS232
        RS_TEMP1
        RS_TEMP2
        TXD
        RXD
        stringlen
        AZ_LO
        AZ_HI
        EL_LO
        EL_HI
        AZkorr_LO
        AZkorr_HI
        ELkorr_LO
        ELkorr_HI
        char_cnt
; Rotor
        AZ_char
        EL_char
        Rotor_Flag      ;U: b0=1,D: b1=1,L: b2=1,R: b3=1,AZ active.b6=0,EL active.b7=0
        Rotor_hyst      ;hyteresis for movements to avoid wear out of rotor mechanics
; FIFO
        fifo_char
        fifo_len
        fifo_start
        fifo_base
        fifo_buffer:.16     ; we need 16 bytes
        zzzz
        endc
;************************************************************************
;***** Declaring the hardware *****	

LCDtris equ     TRISB
LCDport equ     PORTB
ADtris	equ	TRISA
ADC_LO EQU LO
ADC_HI EQU HI
ramadr	equ        AZkorr_LO
#define TXport  PORTA,2
#define TXtris  TRISA,2
#define Data	PORTB,4
#define Clock	PORTB,1
#define CS	PORTA,4


;************************************************************************
;***** Structure of program memory *****

	ORG     0x00	; Reset vector
  	goto    Main

	ORG     0x04	; Interrupt vector
        goto    ISR    ;  interrupt routine

;************************************************************************

Messages		; Beginning of the messages

        movwf   PCL

    			; Display messages

Message1 dt "-- AOT-Rotor -- "	; please do not remove my callsign suffix DL7AOT
END_messages                    ; End of messages

;************************************************************************

shufflechar			; s/r shuffle order of digits.
	movfw	char_cnt	; there are up to 3 digits, least significant 
	addwf	PCL,f		; digit always goes to TEMP2 .
	goto	nullchars
	goto	onechar
	goto	twochars
	return
nullchars
	clrf	TEMP0
	clrf	TEMP1
	clrf	TEMP2
	return
onechar
	movfw	TEMP0
	movwf	TEMP2
	clrf	TEMP0
	clrf	TEMP1
	return
twochars
	movfw	TEMP1
	movwf	TEMP2
	movfw	TEMP0
	movwf	TEMP1
	clrf	TEMP0
	return

;************************************************************************

        #include "bank.inc"	; Assistant files
        #include "lcdbf.inc"
    	#include "print.inc"
        #include "rs232.inc"
	#include "ltc1298.inc"
	#include "lcdv16.inc"
        #include "lcdv08.inc"
        #include "asc2bin.inc"
        #include "math16.inc"
        #include "eeprom.inc"

;************************************************************************
;       Interrupt service fuer RS232

ISR     movwf   w_		; save complete context
        swapf   STATUS, w
        movwf   status_
        bcf     STATUS, RP0
	movfw	PCLATH
	movwf	pclath_
        movfw   FSR
	movwf	fsr_

        RECEIVE

ISRend
	movfw	fsr_		; restore context
        movwf   FSR
	movfw	pclath_
	movwf	PCLATH
        swapf   status_,w
        movwf   STATUS
        swapf   w_,f
        swapf   w_,w
        bcf     INTCON,INTF
        RETFIE

;************************************************************************

Main                    	; Beginning of the program

	bcf PORTB,2
        movlw   0xf0
	movwf	PORTA
	BANK1
	clrf	TRISA		; output on all pins
	movlw	0x01
	movwf	TRISB
	BANK0
        clrw                    ; from EEPROM address 0
        call    get_EElat       ; read 4 corrective values of antenna offset
	movlw	4		; and get hysteresis from location 4
	movwf	EEADR
	call	EE_Read
	movwf	Rotor_hyst
        clrf    AZ_char
        clrf    EL_char
        movlw   0xf0
        movwf   Rotor_Flag

	LCDinit		; LCD initialize
Main1
        RS232init       ; RS232 dtto
	
        PRINT   Messages, Message1, END_messages, Pointer, LCDw

;************************************************************************
Loop
        call    ADC_service
        call    Rotor_Move
        movfw   fifo_len
        btfsc   STATUS,Z
	goto	Loop

; rs232 service dispatcher in case some character received
        call    FIFOget
        movlw   'A'
        subwf   fifo_char,w
        bz      RS_azimut
        movlw   'E'
        subwf   fifo_char,w
        bz      RS_elevat
        movlw   'W'
        subwf   fifo_char,w
        bz      RS_aziele
        movlw   'a'
        subwf   fifo_char,w
        bz      RS_azkorr
        movlw   'e'
        subwf   fifo_char,w
        bz      RS_elkorr
        movlw   'Y'
        subwf   fifo_char,w
        bz      RS_hysterese
        movlw   'M'
        subwf   fifo_char,w
        bz      RS_message
        movlw   'K'
        subwf   fifo_char,w
        bz      show_korrektur
        goto    Loop

;************************************************************************

; easycomm 1 protocol
RS_azimut
        bcf     Rotor_Flag,6    ; AZ active
        LCDcmd  0x88            ; 1stline/8th column
        call    SERV_input
        LCDchar ' '
        LCDchar ' '
        call    A_korr
        cpy16	LO,AZ_LO
        goto    Loop

; adjust for antenna offset
A_korr
	cpy16	AZkorr_LO,LO_TEMP
        call    sub16
        movlw   .16
        movwf   LO_TEMP
        movlw   .14
        movwf   HI_TEMP
        btfsc   HI,7		; only positive values legal, therefore if negative
        call    add16		; countermaesures
        return

; easycomm 1 protocol
RS_elevat
        bcf     Rotor_Flag,7    ; EL aktiv
        LCDcmd  0x80            ; 1st line/1st column
        call    SERV_input
RS_elevat1
        LCDchar ' '
        LCDchar ' '
	cpy16	ELkorr_LO,LO_TEMP
        call    sub16
	cpy16	LO,EL_LO
        btfss   HI,7
        goto    Loop		; positive result, done
	call	add_180
	btfss	HI,7
	goto	ele_null	; positive value results fin stop on 0 degrees
	call	add_180		; it was negative offset, 
	cpy16	LO,EL_LO	; must have been 180 bis 360, now ok again
	goto	Loop
ele_null
        clrf    EL_LO
        clrf    EL_HI
        goto    Loop

add_180
	movlw	.8
	movwf	LO_TEMP
	movlw	.7
	movwf	HI_TEMP
	call	add16
	return

; GS-232 protocol
RS_aziele			
        bcf     Rotor_Flag,6    ; AZ aktiv
        bcf     Rotor_Flag,7    ; EL aktiv
        LCDcmd  0x88
        LCDchar 'A'
        LCDchar 'Z'
        call    SERV_input1
        LCDchar ' '
        LCDchar ' '
        call    A_korr
        cpy16	LO,AZ_LO
        LCDcmd  0x80
        LCDchar 'E'
        LCDchar 'L'
        call    SERV_input1
	goto	RS_elevat1

;my protocol coexistant with easycomm and GS-232
RS_azkorr
        LCDcmd  LCDCLR
        LCDcmd  0x88            ; 1.line/8. column
        call    SERV_input
	cpy16	LO,AZkorr_LO
        goto    Loop


;my protocol coexistant with easycomm and GS-232
RS_elkorr
        LCDcmd  0x80            ; 1.line/1. column
        call    SERV_input
	cpy16	LO,ELkorr_LO
	clrw			; EEPROM location 0
        call    put_EElat       ;save 4 corrective values in EEprom 
        goto    Loop


;my protocol coexistant with easycomm and GS-232
RS_hysterese
        LCDcmd  0x80            ; 1.line/1. column
	movlw	4
	movwf	EEADR
        call    SERV_input
	movfw	LO
	movwf	Rotor_hyst
        call    EE_Write
        goto    Loop


;my protocol coexistant with easycomm and GS-232
RS_message
        call    GET_next        ; part two, no prefix
        LCDline 1
        movlw   .16
        movwf   stringlen
msg_loop
        call    GET_next
        movlw   ';'
        subwf   fifo_char,w
        bz      blank
        movfw   fifo_char
        LCDw
        decfsz  stringlen,f
        goto    msg_loop
        goto    Loop
blank
        LCDchar ' '
        decfsz  stringlen,f
        goto    blank
        goto    Loop



GET_next
        movfw   fifo_len
        btfsc   STATUS,Z
        goto    GET_next
        call    FIFOget		; character still in w
        sublw   '.'             ; ignore this but do not remove, it will not matter ( hopefully )
        return

SERV_input  movfw   fifo_char	; 1st character already here and checked
        LCDw			; send to display
        call    GET_next	; 2nd character
        movfw   fifo_char	; will not be checked for legal syntax
        LCDw
SERV_input1			; here we go with GS-232 protocol
	clrf TEMP3		; max 3 digits allowed,plus decimal dot
	clrf	char_cnt	; and one decimal digit
;
        call    GET_next
        bz      SERV_fertig     ;'.' received
	incf	char_cnt,f	;1st digit	
        movfw   fifo_char
        movwf   TEMP0
;
        call    GET_next
        bz      SERV_fertig     ;'.' received
	incf	char_cnt,f	;2nd digit	
        movfw   fifo_char
        movwf   TEMP1
;
        call    GET_next
        bz      SERV_fertig     ;'.' received
	incf	char_cnt,f	;3 digits received	
        movfw   fifo_char
        movwf   TEMP2
        call    GET_next
        bnz      SERV_exit	;no '.' received, exit
SERV_fertig
        call    GET_next
        movfw   fifo_char
        andlw   0x0f            ; internal arithmetic accuracy is 10th degrees
        movwf   TEMP3           ; to make use of A/D precision
SERV_exit
        call    shufflechar
        call    ASC2BIN         ; convert to binary
        call    LCDval16	; display value, digits preceding decimal dot only
        return

;************************************************************************

ADC_service
;Azimuth
;        LCDline 2
        LCDcmd  0xc8            ; 2.line/8. column
        LCDchar 'a'
        LCDchar 'z'
        bcf     ADC_Var,7	; A/D-channel 0 is azimuth
        call    LTC98;
        cpy16   AZkorr_LO,LO_TEMP
        call    ADC_korr	; take care of antenna offset
        LCDcmd  0xce            ; 2.line/15. column
; Auswertung fuer Rotorsteuerung
	cpy16	AZ_LO,LO_TEMP
        call    sub16           ; measured value minus set value
        movlw   'R'             ; assume negative: rotate right
        btfsc   HI,7            ; result negative ?
        goto    AZ_right        ; yes
        clrf    HI_TEMP         ; check for stop
        movfw   Rotor_hyst      ; hysteresis in 10th of degrees
        movwf   LO_TEMP
        call    sub16
        movlw   'L'             ; wassume positive: rotate left
        btfsc   HI,7            ; result negative ?
        movlw   'A'             ; good range: stop azimuth movement
AZ_right
        movwf   AZ_char
        LCDw
;
Elevation
        LCDcmd  0xc0            ; 2.line/1. column
        LCDchar 'e'
        LCDchar 'l'
	bsf	ADC_Var,7	; A/D-channel 1 is elevation
        call    LTC98
        shr16   LO
        cpy16   ELkorr_LO,LO_TEMP
        call    ADC_korr	; take care of antenna offset
        LCDcmd  0xc6            ; 2.line/7. column
; Auswertung fuer Rotorsteuerung
	cpy16	EL_LO,LO_TEMP
        call    sub16           ; measured value minus set value
        movlw   'U'             ; assume negative: move up
        btfsc   HI,7            ; result negative ?
        goto    EL_up           ; yes
        clrf    HI_TEMP         ; check for stop
        movfw   Rotor_hyst      ; hysteresis in 10th of degrees
        movwf   LO_TEMP
        call    sub16
        movlw   'D'             ; assume positive: move down
        btfsc   HI,7            ; result negative ?
        movlw   'E'             ; good range: stop elevation movement
EL_up
        movwf   EL_char
        LCDw
	return

ADC_korr			; we insist on displaying real compass direction
        cpy16     LO,ADLO_sav	; therefore adjust displayed value for
        call    add16		; antenna offset ( = position at zero stop )
        movlw   .16
        movwf   LO_TEMP
        movlw   .14
        movwf   HI_TEMP
        call    cmpless16	; mathematics are modulo 360
        btfss   HI_RESULT,7	; therefore in case
        call    sub16		; adjust
        LCDval_16               ; displayed data only
        cpy16     ADLO_sav,LO	; we continue mathematics with ADC-values
        return

;************************************************************************

Rotor_Move
        btfsc   Rotor_Flag,6            ; no L/R movement, if not active
        goto    Rotor_Move_EL
	bcf	Rotor_Flag,3
	bcf	Rotor_Flag,2
        movlw   'R'
        subwf   AZ_char,w
        btfsc   STATUS,Z
        bsf     Rotor_Flag,3            ; move right
        movlw   'L'
        subwf   AZ_char,w
        btfsc   STATUS,Z
        bsf     Rotor_Flag,2            ; move left
        movlw   'A'
        subwf   AZ_char,w
        btfsc   STATUS,Z
        bsf     Rotor_Flag,6            ; declare AZ not active
Rotor_Move_EL
        btfsc   Rotor_Flag,7            ; no U/D movement, if not active
        goto Rotor_Move_c
	bcf	Rotor_Flag,1
	bcf	Rotor_Flag,0
        movlw   'D'
        subwf   EL_char,w
        btfsc   STATUS,Z
        bsf     Rotor_Flag,1            ; move down
        movlw   'U'
        subwf   EL_char,w
        btfsc   STATUS,Z
        bsf     Rotor_Flag,0            ; move up
        movlw   'E'
        subwf   EL_char,w
        btfsc   STATUS,Z
        bsf     Rotor_Flag,7            ; declare EL not active
Rotor_Move_c
        movfw   Rotor_Flag
        movwf   PORTA			; send new command to rotor
        return


;************************************************************************
show_korrektur
	LCDline	1
        LCDchar 'e'
        LCDchar 'k'
	cpy16	ELkorr_LO,LO
        LCDval_16
        LCDchar 'a'
        LCDchar 'k'
	cpy16	AZkorr_LO,LO
        LCDval_16
        LCDchar 'h'
        movfw   Rotor_hyst
        LCDval_08
        goto    Loop

;************************************************************************
; set my default EEPROM:
; location  0: azimut offset of antenna, 16bit double precision
; location  2: elevation offset of antenna, 16bit double precision
; location  4: hysteresis of antenna movement ( in tenth of degrees, single precision
;
        org 0x2100
EE_str0 DE      0x00,0x00,0x00,0x00,.20 ; azimut low/high,elevation low/high-offset,hysteresis
EE_str1 DE      "R","o","t","o","r"," ","I","n","t","e","r","f","a","c","e"," ",0
;************************************************************************

	End		; End of program


